from .client import ZXArtClient
from .common import Entity, Language, Order, OrderSettings
from .models import Image, Tune

__all__ = [
    "Entity",
    "Image",
    "Language",
    "Order",
    "OrderSettings",
    "Tune",
    "ZXArtClient",
]



import asyncio
import logging

from .client import ZXArtClient
from .common import Entity, Language, Order

logging.basicConfig(level=logging.DEBUG)


async def main():
    async with ZXArtClient(language=Language.RUSSIAN, limit=10) as cli:
        result = await cli.api(
            Entity.TUNE,
            order=Order.MOST_RECENT,
            min_party_place=1,
            years=range(2023, 2026),
        )

        for x in result.result:
            print(x)


asyncio.run(main())



import dataclasses as dc
import datetime as dt
import html
import re
from decimal import Decimal
from typing import Annotated
from urllib.parse import unquote

from mashumaro.config import BaseConfig
from mashumaro.mixins.dict import DataClassDictMixin
from mashumaro.types import Discriminator

from .common import Entity

_RE_DESCRIPTION = re.compile(r"<pre>(.*)</pre>", re.DOTALL)

type HtmlStr = Annotated[str, "HtmlStr"]
"""Строка с экранированными символами HTML"""

type UrlStr = Annotated[str, "UrlStr"]
"""Строка с экранированными символами URL"""


def _unescape(value: str) -> str:
    value = html.unescape(value)
    if m := _RE_DESCRIPTION.fullmatch(value):
        return m.group(1)
    return value


def _duration(value: str) -> dt.timedelta:
    m = map(float, reversed(value.split(":")))
    s = sum(x * k for x, k in zip(m, [1, 60, 3600]))
    return dt.timedelta(seconds=s)


def _date(value: str) -> dt.date:
    return dt.datetime.strptime(value, "%d.%m.%Y").date()


@dc.dataclass
class ProductCategory:
    """Категория"""

    id: int
    """Идентификатор"""
    title: HtmlStr
    """Название"""


@dc.dataclass(kw_only=True)
class EntityBase:
    """Базовый класс сущности API"""

    id: int
    """Идентификатор"""
    title: HtmlStr | None = None
    """Название"""
    url: UrlStr
    """URL страницы с описанием"""
    created: dt.datetime
    """Дата и время создания записи"""
    modified: dt.datetime
    """Дата и время последнего изменения"""

    class Config(BaseConfig):
        aliases = {
            "author_id": "authorId",
            "author_ids": "authorIds",
            "created": "dateCreated",
            "duration": "time",
            "end_date": "endDate",
            "filename": "originalFileName",
            "image_url": "imageUrl",
            "import_ids": "importIds",
            "modified": "dateModified",
            "mp3_url": "mp3FilePath",
            "name": "realName",
            "num_images": "picturesQuantity",
            "num_tunes": "tunesQuantity",
            "original_url": "originalUrl",
            "party_id": "partyId",
            "party_place": "partyPlace",
            "start_date": "startDate",
            "title_internal": "internalTitle",
        }

        serialization_strategy = {
            dt.date: {"deserialize": _date},
            dt.datetime: {"deserialize": dt.datetime.fromtimestamp},
            dt.timedelta: {"deserialize": _duration},
            HtmlStr: {"deserialize": _unescape},
            UrlStr: {"deserialize": unquote},
        }


@dc.dataclass(kw_only=True)
class MediaBase(EntityBase):
    """Базовый класс медиафайла"""

    party_id: int | None = None
    """Идентификатор мероприятия"""
    compo: str | None = None
    """Тип"""
    party_place: int | None = None
    """Занятое место на мероприятии"""
    author_ids: list[int]
    """Идентификаторы авторов"""
    tags: list[str] | None = None
    """Теги"""
    type: str | None = None
    """Тип файла"""
    rating: Decimal
    """Рейтинг"""
    year: int | None = None
    """Год написания"""
    description: HtmlStr | None = None
    """Описание"""
    original_url: UrlStr | None = None
    """URL оригинального файла"""


@dc.dataclass(kw_only=True)
class Tune(MediaBase):
    """Мелодия"""

    title_internal: HtmlStr | None = None
    """Внутреннее название"""
    duration: dt.timedelta | None = None
    """Длительность"""
    plays: int | None = None
    """Кол-во прослушиваний"""
    filename: UrlStr | None = None
    """Имя оригинального файла"""
    mp3_url: UrlStr | None = None
    """URL файла MP3"""


@dc.dataclass(kw_only=True)
class Image(MediaBase):
    """Изображение"""

    image_url: UrlStr | None = None
    """URL изображения"""
    views: int | None = None
    """Кол-во просмотров"""


@dc.dataclass
class ImportID:
    """Импортированные идентификаторы автора на сторонних ресурсах."""

    zxaaa: str | None = None
    """https://zxaaa.net/"""
    demozoo: str | None = None
    """https://demozoo.org/"""
    pouet: str | None = None
    """https://www.pouet.net/"""
    spectrumcomputing: str | None = None
    """https://spectrumcomputing.co.uk/"""
    worldofspectrum: str | None = None
    """https://worldofspectrum.net/"""
    vtrd: str | None = None
    """https://vtrd.in/"""
    zxdemo: str | None = None
    """https://zxdemo.org/"""
    speccy: str | None = None
    """https://speccy.info/"""

    class Config(BaseConfig):
        aliases = {
            "zxaaa": "3a",
            "demozoo": "dzoo",
            "spectrumcomputing": "sc",
            "worldofspectrum": "wos",
            "vtrd": "vt",
            "zxdemo": "zxd",
            "speccy": "swiki",
        }


@dc.dataclass(kw_only=True)
class AuthorAlias(EntityBase):
    """Модель псевдонима автора"""

    author_id: int | None = None
    """Идентификатор настоящего автора"""
    import_ids: ImportID | None = None
    """Идентификаторы на других ресурсах"""
    start_date: dt.date | None = None
    """Дата начала действия"""
    end_date: dt.date | None = None
    """Дата окончания действия"""


@dc.dataclass(kw_only=True)
class Author(EntityBase):
    """Модель категории"""

    name: str | None = None
    """Настоящее имя"""
    country: str | None = None
    """Страна"""
    city: str | None = None
    """Город"""
    num_images: int = 0
    """Количество изображений"""
    num_tunes: int = 0
    """Количество мелодий"""
    aliases: list[int] | None = None
    """Идентификаторы псевдонимов"""
    import_ids: ImportID | None = None
    """Идентификаторы на других ресурсах"""


@dc.dataclass
class ApiResponse[T](DataClassDictMixin):
    """Модель ответа на запросы"""

    total: int
    """Всего записей в базе данных"""
    start: int
    """Начальный индекс"""
    limit: int
    """Ограничение"""
    result: list[T]
    """Данные ответа"""

    class Config(BaseConfig):
        aliases = {"total": "totalAmount"}
        discriminator = Discriminator(field="entity", include_subtypes=True)
        lazy_compilation = True


@dc.dataclass
class AuthorResponse(ApiResponse[Author]):
    entity = Entity.AUTHOR


@dc.dataclass
class AuthorAliasResponse(ApiResponse[AuthorAlias]):
    entity = Entity.AUTHOR_ALIAS


@dc.dataclass
class ProductCategoryResponse(ApiResponse[ProductCategory]):
    entity = Entity.PRODUCT_CATEGORY


@dc.dataclass
class TuneResponse(ApiResponse[Tune]):
    entity = Entity.TUNE


@dc.dataclass
class ImageResponse(ApiResponse[Image]):
    entity = Entity.IMAGE



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, overload

import aiohttp
import orjson
import yarl

from .common import Entity, Language, Order, process_filters
from .models import ApiResponse

if TYPE_CHECKING:
    from typing import Any, Literal, Unpack

    from .common import CommonOptions, OrderSettings
    from .image import ImageParams
    from .models import Author, AuthorAlias, Image, ProductCategory, Tune
    from .tune import TuneParams

_LOGGER = logging.getLogger(__name__)

# Опции по-умолчанию
_DEFAULT_ORDER = Order.MOST_RECENT
_DEFAULT_LANGUAGE = Language.RUSSIAN
_DEFAULT_LIMIT = 60

_BASE_URL = yarl.URL("https://zxart.ee/api/")
"""Базовый URL API"""


class ZXArtApiError(Exception):
    """Ошибка API"""


class ZXArtClient:
    """Клиент ZXArt"""

    _cli: aiohttp.ClientSession
    _language: Language
    _limit: int
    _order: Order | OrderSettings

    def __init__(
        self,
        *,
        language: Language | None = None,
        limit: int | None = None,
        order: Order | OrderSettings | None = None,
        session: aiohttp.ClientSession | None = None,
    ) -> None:
        """
        Инициализирует клиент параметрами запроса `по-умолчанию`.

        Параметры:
        - `language`: язык переводимых полей сущностей. По-умолчанию: `русский`.
        - `limit`: ограничение по количеству записей. По-умолчанию: `60`.
        - `order`: порядок сортировки результата. По-умолчанию: `сначала последние`.
        - `session`: пользовательская сессия `aiohttp.ClientSession`.
        """

        self._language = language or _DEFAULT_LANGUAGE
        self._limit = limit or _DEFAULT_LIMIT
        self._order = order or _DEFAULT_ORDER
        self._cli = session or aiohttp.ClientSession()
        self._close_connector = not session

    async def __aenter__(self):
        return self

    def __aexit__(self, exc_type, exc_value, traceback):
        return self.close()

    async def close(self):
        """Закрытие"""

        if self._close_connector:
            await self._cli.close()

    @overload
    async def api(
        self,
        entity: Literal[Entity.AUTHOR],
        **kwargs: Unpack[CommonOptions],
    ) -> ApiResponse[Author]: ...

    @overload
    async def api(
        self,
        entity: Literal[Entity.AUTHOR_ALIAS],
        **kwargs: Unpack[CommonOptions],
    ) -> ApiResponse[AuthorAlias]: ...

    @overload
    async def api(
        self,
        entity: Literal[Entity.PRODUCT_CATEGORY],
        **kwargs: Unpack[CommonOptions],
    ) -> ApiResponse[ProductCategory]: ...

    @overload
    async def api(
        self,
        entity: Literal[Entity.TUNE],
        **kwargs: Unpack[TuneParams],
    ) -> ApiResponse[Tune]:
        """
        Запрос мелодий по параметрам:

        Общие параметры (значения по-умолчанию из клиента):
        - `language`: язык переводимых полей сущностей.
        - `start`: стартовая позиция курсора запроса.
        - `limit`: ограничение по количеству записей.
        - `order`: порядок сортировки результата.

        Фильтры:
        - `id`: идентификатор.
        - `author_id`: идентификатор автора.
        - `title`: контекст наименования.
        - `years`: год(а) издания (поддерживает `Iterable`).
        - `min_rating`: минимальный рейтинг.
        - `min_party_place`: минимальное место на мероприятии.
        - `tags_include`: включает тег(и) (поддерживает `Iterable`).
        - `tags_exclude`: не включает тег(и) (поддерживает `Iterable`).
        - `format`: формат мелодии.
        - `format_group`: формат группы мелодии.
        """

    @overload
    async def api(
        self,
        entity: Literal[Entity.IMAGE],
        **kwargs: Unpack[ImageParams],
    ) -> ApiResponse[Image]:
        """
        Запрос изображений по параметрам:

        Общие параметры (значения по-умолчанию из клиента):
        - `language`: язык переводимых полей сущностей.
        - `start`: стартовая позиция курсора запроса.
        - `limit`: ограничение по количеству записей.
        - `order`: порядок сортировки результата.

        Фильтры:
        - `id`: идентификатор.
        - `author_id`: идентификатор автора.
        - `title`: контекст наименования.
        - `years`: год(а) издания (поддерживает `Iterable`).
        - `min_rating`: минимальный рейтинг.
        - `min_party_place`: минимальное место на мероприятии.
        - `tags_include`: включает тег(и) (поддерживает `Iterable`).
        - `tags_exclude`: не включает тег(и) (поддерживает `Iterable`).
        """

    async def api(self, entity: Entity, **kwargs: Any) -> ApiResponse:
        if kwargs:
            process_filters(entity, kwargs)

        kwargs.setdefault("language", self._language)
        kwargs.setdefault("limit", self._limit)
        kwargs.setdefault("order", self._order)
        kwargs["export"] = entity

        url = _BASE_URL.joinpath(*(f"{k}:{v}" for k, v in kwargs.items()))

        _LOGGER.debug("API request URL: %s", url)

        async with self._cli.get(url) as x:
            json: dict[str, Any] = await x.json(loads=orjson.loads)

        if json.pop("responseStatus") != "success":
            raise ZXArtApiError("API request error.")

        json["result"] = json.pop("responseData")[entity]
        json["entity"] = entity

        return ApiResponse.from_dict(json)

    async def author(self, author_id: int) -> Author | None:
        """Запрос автора по идентификатору."""

        if x := (await self.api(Entity.AUTHOR, id=author_id)).result:
            return x[0]



import dataclasses as dc
from enum import Enum, StrEnum
from typing import Any, Iterable, Literal, TypedDict


@dc.dataclass(frozen=True, slots=True)
class OrderSettings:
    """Параметры сортировки."""

    field: Literal[
        "year",
        "plays",
        "title",
        "place",
        "date",
        "votes",
        "commentsAmount",
    ]
    """Поле сортировки."""

    order: Literal["asc", "desc", "rand"] = "desc"
    """Порядок сортировки. По-умолчанию: убывающий."""

    def __str__(self):
        return f"{self.field},{self.order}"


class Order(Enum):
    """Часто использованные шаблоны сортировки."""

    TOP_RATED = OrderSettings("votes")
    """Самые рейтинговые"""
    MOST_PLAYED = OrderSettings("plays")
    """Самые прослушиваемые"""
    MOST_RECENT = OrderSettings("date")
    """Недавно загруженные"""
    TOP_PLACED = OrderSettings("place", "asc")
    """Самые оцененные на мероприятиях"""
    MOST_COMMENTED = OrderSettings("commentsAmount")
    """Самые комментируемые"""

    def __str__(self):
        return str(self.value)


class Language(StrEnum):
    """Предпочитаемый язык переводимых полей ответа."""

    ENGLISH = "eng"
    """Английский"""
    RUSSIAN = "rus"
    """Русский"""
    SPANISH = "spa"
    """Испанский"""


class Entity(StrEnum):
    """Сущности поддерживаемые API."""

    AUTHOR = "author"
    """Автор"""
    AUTHOR_ALIAS = "authorAlias"
    """Псевдоним автора"""
    GROUP = "group"
    """Группа"""
    GROUP_ALIAS = "groupAlias"
    """Псевдоним группы"""
    PRODUCT = "zxProd"
    """Продукт"""
    PRODUCT_CATEGORY = "zxProdCategory"
    """Категория продукта"""
    RELEASE = "zxRelease"
    """Релиз"""
    IMAGE = "zxPicture"
    """Изображение"""
    TUNE = "zxMusic"
    """Мелодия"""


class CommonOptions(TypedDict, total=False):
    """Общие опции запроса"""

    language: Language
    """Язык переводимых полей ответа."""
    limit: int
    """Ограничение ответа."""
    order: OrderSettings | Order
    """Порядок сортировки."""
    start: int
    """Индекс начальной записи выборки."""
    id: int
    """Фильтр: идентификатор сущности"""


class MediaParams(CommonOptions, total=False):
    """Опции фильтра"""

    author_id: int
    """Фильтр: идентификатор автора"""
    title: str
    """Фильтр: содержание наименования"""
    years: Iterable[int] | int
    """Фильтр: годы публикации"""
    min_rating: float
    """Фильтр: минимальный рейтинг"""
    min_party_place: int
    """Фильтр: минимальное место на мероприятии"""
    tags_include: Iterable[str] | str
    """Фильтр: с тегами"""
    tags_exclude: Iterable[str] | str
    """Фильтр: без тегов"""


_FILTER_MAP = {
    "author_id": "authorId",
    "compo": "Compo",
    "format_group": "FormatGroup",
    "format": "Format",
    "has_inspiration": "Inspiration",
    "has_stages": "Stages",
    "id": "Id",
    "min_party_place": "MinPartyPlace",
    "min_rating": "MinRating",
    "tags_exclude": "TagsExclude",
    "tags_include": "TagsInclude",
    "title": "TitleSearch",
    "type": "Type",
    "years": "Year",
}


def process_filters(entity: Entity, kwargs: dict[str, Any]) -> None:
    """Обработка параметров фильтров"""

    filters: list[str] = []

    for arg in tuple(kwargs):
        if (k := _FILTER_MAP.get(arg)) is None:
            continue

        v = kwargs.pop(arg)

        if not isinstance(v, str) and isinstance(v, Iterable):
            v = ",".join(map(str, v))

        if k[0].isupper():
            k = entity + k

        filters.append(f"{k}={v}")

    if filters:
        kwargs["filter"] = ";".join(filters)



from typing import Literal

from .common import MediaParams

ImageFormat = Literal[
    "AKS",
    "AS0",
    "ASC",
    "AY",
    "CHI",
    "CHP",
    "COP",
    "DMM",
    "DST",
    "ET1",
    "FTC",
    "FUR",
    "GTR",
    "MOD",
    "MP3",
    "MTC",
    "PDT",
    "PSC",
    "PSG",
    "PSM",
    "PT1",
    "PT2",
    "PT3",
    "SQD",
    "SQT",
    "ST1",
    "ST3",
    "STC",
    "STP",
    "STR",
    "TF0",
    "TFC",
    "TFD",
    "TFE",
    "TS",
    "VGM",
    "VTX",
    "WAV",
    "YM",
]


ImageFormatGroup = Literal[
    "ay",
    "beeper",
    "digitalbeeper",
    "beeperdigitalbeeper",
    "digitalay",
    "ts",
    "fm",
    "tsfm",
    "aybeeper",
    "aydigitalay",
    "aycovox",
    "saa",
]


class ImageParams(MediaParams, total=False):
    type: ImageFormat
    format_group: ImageFormatGroup
